/*
 *   AkResultMessage.H
 *   =================
 *
 *   An AkResultMessage is the type of message output by
 *   the Akaroa star. It contains an array of AkResult
 *   structures, one for each parameter.
 *
 *   The AkResultMessage::param method provides 1-based
 *   index access to the array.
 *
 *   An AkResultMessage prints itself as a Tcl list with
 *   one element for each parameter. Each element is a list
 *   of six items:
 *
 *      {count trans mean variance delta conf}
 */

#ifndef AkResultMessage_H
#define AkResultMessage_H

class AkResult;

class AkResultMessage: public Message {
public:
  int numParams;
  AkResult *params;

  AkResultMessage(int numParams);
  virtual ~AkResultMessage();

  const AkResult &param(int i) const;
  AkResult &param(int i);

  virtual StringList print() const;
  virtual Message *clone() const;
  const char *dataType();
  int isA(const char *) const;
};

#endif

