static const char file_id[] = "DEAkObservation.pl";
// .cc file generated from DEAkObservation.pl by ptlang

#ifdef __GNUG__
#pragma implementation
#endif

#include "DEAkObservation.h"
#include "iostream.h"
#include "strstream.h"
#include "SimControl.h"
#include "Matrix.h"
#include "AkResultMessage.H"
#include "akaroa/result.H"

const char *star_nm_DEAkObservation = "DEAkObservation";

const char* DEAkObservation :: className() const {return star_nm_DEAkObservation;}

ISA_FUNC(DEAkObservation,DEStar);

Block* DEAkObservation :: makeNew() const { LOG_NEW; return new DEAkObservation;}

# line 120 "DEAkObservation.pl"
extern ACG *gen;

    AkResultMessage::AkResultMessage(int n) {
      numParams = n;
      params = new AkResult[n];
    }

    AkResultMessage::~AkResultMessage() {
      delete[] params;
    }

    const AkResult &AkResultMessage::param(int i) const {
      return params[i-1];
    }

    AkResult &AkResultMessage::param(int i) {
      return params[i-1];
    }

    StringList AkResultMessage::print() const {
      strstream str;
      for (int i = 1; i <= numParams; i++) {
	const AkResult &r = param(i);
	str << "{" 
	    << r.count << " " 
	    << r.trans << " "
	    << r.mean << " "
	    << r.variance << " "
	    << r.delta << " "
	    << r.conf
	    << "} ";
      }
      str << ends;
      StringList sl(str.str());
      str.freeze(0);
      return sl;
    }

    Message *AkResultMessage::clone() const {
      AkResultMessage *copy = new AkResultMessage(numParams);
      for (int i = 1; i <= numParams; i++)
	copy->param(i) = param(i);
      return copy;
    }
	
    const char *AkResultMessage::dataType() {return "AkResultMessage";}

    ISA_FUNC(AkResultMessage, Message);
      
    class PtolemyReporter: public AkReporter {
      AkResultMessage *msg;
      DEAkObservation *star;

    public:
      PtolemyReporter(DEAkObservation *s) {
	star = s;
	msg = 0;
      }

      virtual void Begin(int n) {
	msg = new AkResultMessage(n);
      }

      virtual void Report(int n, const AkResult &r) {
	msg->param(n) = r;
      }

      virtual void End() {
	star->completionTime = star->arrivalTime;
	star->output.put(star->completionTime) << *msg;
	msg = 0;
      }
    };

    class AkRNG : public ACG {

      long spareBits;
      int numSpareBits;

    public:

      AkRNG() {
	numSpareBits = 0;
      }

      virtual _G_uint32_t asLong() {
	if (numSpareBits == 0) {
	  spareBits = AkRandom();
	  numSpareBits = 31;
	}
	long bits = (AkRandom() << 1) | (spareBits & 1);
	spareBits >>= 1;
	--numSpareBits;
	return bits ^ (bits << 1);
      }

    };

DEAkObservation::DEAkObservation ()
{
	setDescriptor("Submits observations to Akaroa and outputs final simulation results.");
	addPort(input.setPort("input",this,FLOAT));
	addPort(output.setPort("output",this,MESSAGE));
	addState(precision.setState("precision",this,"0.05","precision"));
	addState(confidence.setState("confidence",this,"0.95","confidence"));
	addState(debug.setState("debug",this,"FALSE","debug"));

# line 220 "DEAkObservation.pl"
oa = 0;
    reporter = new PtolemyReporter(this);
    ak_rng = new AkRNG;
    stopTime = 0;
}

DEAkObservation::~DEAkObservation() {
# line 227 "DEAkObservation.pl"
delete oa;
    delete reporter;
}

void DEAkObservation::setup() {
# line 232 "DEAkObservation.pl"
if (oa) {
      delete oa;
      oa = 0;
    }
    count = 0;
    if (gen != ak_rng) {
      if (debug)
	Error::message("Installing Akaroa random number generator");
      gen = ak_rng;
    }
    else {
      if (debug)
	Error::message("Akaroa random number generator installed");
    }
    Environment *env = DefaultEnvironment();
    env->SetReal("Precision", precision);
    env->SetReal("Confidence", confidence);
    stopFlag = FALSE;
}

void DEAkObservation::go() {
# line 253 "DEAkObservation.pl"
if (stopFlag) {
      if (arrivalTime > stopTime)
	SimControl::requestHalt();
      return;
    }
    if (!oa) {
      int numParameters = input.numberPorts();
      if (debug) {
	char buf[200];
	sprintf(buf, 
		"Creating an ObservationAnalyser for %ld parameters"
		" with reporter %p",
		numParameters, reporter); 
	Error::message(buf);
      }
      oa = new ObservationAnalyser(numParameters, reporter);
    }
    MPHIter nextport(input);
    InDEPort *port;
    int i = 1;
    while (port = (InDEPort*)(nextport++)) {
      int paramNumber = i++;
      if (port->dataNew) {
	double value = port->get();
	if (debug) {
	  ostrstream str;
	  str << "*AkObservation: Param " << paramNumber
	       << ", observation " << ++count << ": " << value << endl;
	  Error::message(str.str());
	  str.freeze(0);
	}
	oa->ProcessObservation(paramNumber, value);
      }
    }
    if (oa->SimulationOver()) {
      oa->Report();
      Error::message("Akaroa is stopping the simulation.");
      stopFlag = TRUE;
      stopTime = arrivalTime;
    }
}

// prototype instance for known block list
static DEAkObservation proto;
static RegisterBlock registerBlock(proto,"AkObservation");
